package ca.uwaterloo.gp.editorlist;

/**********************************************************************
Copyright (c) 2002 IBM Corp.  All rights reserved.
This file is made available under the terms of the Common Public License v1.0
which accompanies this distribution, and is available at
http://www.eclipse.org/legal/cpl-v10.html
*********************************************************************/

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPageListener;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

/**
 * The editor part listener reacts to editor opening,
 * closing, and activation, and mouse selection. It ensures that editors
 * are added, removed, and selected in the editor list
 * when these events occur. It also listens to editor reference
 * changes to ensure that editors are rendered in the editor 
 * list with the appropriate images (error overlays).
 */
public class EditorListListener implements IPartListener, IPageListener, IPropertyListener, MouseListener, KeyListener, ILabelProviderListener {
	
	private EditorList fView;
	
	public EditorListListener(EditorList view) {
		fView= view;
	}
	/**
	 * @see IPartListener#partActivated(IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part) {
		if (part instanceof IEditorPart) {
			fView.editorSelected((IEditorPart)part);
		}
	}
	/**
	 * @see IPartListener#partClosed(IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part) {
		if (part instanceof IEditorPart) {
			fView.editorClosed((IEditorPart)part);
		}
	}
	/**
	 * @see IPartListener#partOpened(IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {
		if (part instanceof IEditorPart) {
			fView.addEditorToViewer((IEditorPart)part);
		} else if (part == fView) {
			fView.refreshContents();
		}
	}
	/**
	 * @see IPartListener#partDeactivated(IWorkbenchPart)
	 */
	public void partDeactivated(IWorkbenchPart part) {
	}
	/**
	 * @see IPartListener#partBroughtToTop(IWorkbenchPart)
	 */
	public void partBroughtToTop(IWorkbenchPart part) {
		if (part instanceof IEditorPart) {
			fView.editorSelected((IEditorPart)part);
		}
	}
	/**
	 * @see IPropertyListener#propertyChanged(java.lang.Object, int)
	 */
	public void propertyChanged(Object source, int propId) {
		if (source instanceof IEditorPart && (propId == IWorkbenchPart.PROP_TITLE ||
					propId == IEditorPart.PROP_DIRTY)) {
			fView.refreshLabel((IEditorPart) source);
		}
	}
	/**
	 * @see MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 * 
	 * When an editor is selected, bring it to the top.
	 */
	public void mouseDown(MouseEvent event) {
		fView.bringToTopSelectedEditor();
	}
	/**
	 * @see MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 * 
	 * When an editor is double-clicked, activate it.
	 */
	public void mouseDoubleClick(MouseEvent event) {
		fView.activateSelectedEditor();
	}
	/**
	 * @see MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 * 
	 * Do nothing.
	 */
	public void mouseUp(MouseEvent event) {
	}
	/**
	 * @see org.eclipse.ui.IPageListener#pageActivated(org.eclipse.ui.IWorkbenchPage)
	 */
	public void pageActivated(IWorkbenchPage page) {
		fView.refreshContents();
	}
	/**
	 * @see org.eclipse.ui.IPageListener#pageClosed(org.eclipse.ui.IWorkbenchPage)
	 */
	public void pageClosed(IWorkbenchPage page) {
	}
	/**
	 * @see org.eclipse.ui.IPageListener#pageOpened(org.eclipse.ui.IWorkbenchPage)
	 */
	public void pageOpened(IWorkbenchPage page) {
		fView.refreshContents();
	}
	/**
	 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
	}
	/**
	 * Handles key events in viewer. Hotkeys for this view include:
	 * <ul>
	 * <li>DELETE - close selected editors</li>
	 * <li>F5 - refresh the view's contents</li>
	 * </ul>
	 */
	public void keyReleased(KeyEvent event) {
		if (event.stateMask == 0) {
			if (event.character == SWT.DEL) {
				fView.closeSelectedEditors();
			} else if (event.keyCode == SWT.F5) {
				fView.refreshContents();
			} else if (event.character == SWT.CR) {
				fView.bringToTopSelectedEditor();
			}
		}
	}
	/**
	 * @see org.eclipse.jface.viewers.ILabelProviderListener#labelProviderChanged(org.eclipse.jface.viewers.LabelProviderChangedEvent)
	 */
	public void labelProviderChanged(LabelProviderChangedEvent event) {
		Object[] elements= event.getElements();
		if (elements == null) {
			fView.refreshContents();
			return;
		}
		for (int i = 0; i < elements.length; i++) {
			Object element= elements[i];
			if (element instanceof EditorListElement) {
				IEditorPart editor= ((EditorListElement) element).getEditorReference().getEditor(true);
				if (editor != null) {
					fView.refreshLabel(editor);
				}
			}
		}
	}
}